#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int alarm_on = 0, pos = 0, recv = 0, emergency = 0;
char buf[500];
char phoneNo[] = "+40721700994";

void USART0_init()
{
    /* seteaza baud rate la 9600 */
    UBRR0H = 0;
    UBRR0L = 103;

    /* porneste transmitatorul */
    UCSR0B = (1<<TXEN0) | (1<<RXEN0);
    UCSR0B |= (1<<RXCIE0);

    /* seteaza formatul frame-ului: 8 biti de date, 1 biti de stop */
    UCSR0C = (3<<UCSZ00);
    UCSR0C &= ~(1<<USBS0);
}

/*
 * Functie ce transmite un caracter prin USART
 *
 * @param data - caracterul de transmis
 */
void USART0_transmit(char data)
{
    /* asteapta pana bufferul e gol */
    while(!(UCSR0A & (1<<UDRE0)));

    /* pune datele in buffer; transmisia va porni automat in urma scrierii */
    UDR0 = data;
}

/*
 * Functie ce primeste un caracter prin USART
 *
 * @return - caracterul primit
 */
char USART0_receive()
{
    /* asteapta cat timp bufferul e gol */
    while(!(UCSR0A & (1<<RXC0)));

    /* returneaza datele din buffer */
    return UDR0;
}

/*
 * Functie ce transmite un sir de caractere prin USART
 *
 * @param data - sirul (terminat cu '\0') de transmis
 */
void USART0_print(const char *data)
{
    while(*data != '\0')
        USART0_transmit(*data++);
}


char* get_msg(char *sms)
{
	int i;
	char *msg = (char *) calloc(50, sizeof(char));

	for(i = strlen(sms) - 2; i >= 0; i--) {
		if(sms[i] == 0x0A) {
			strncpy(msg, sms + i + 1, strlen(sms) - 3 - i);
			break;
		}
	}

	return msg;
}

char* get_no(char *sms)
{
	int i, occ = 0, len = strlen(sms);
	char *msg = (char *) calloc(50, sizeof(char));

	for(i = 0; i < len; i++) {
		if(sms[i] == '\"') {
			occ++;
		}
		if(occ == 3) {
			strncpy(msg, sms + i + 1, 12);
			msg[14] = '\0';
			break;
		}
	}

	return msg;
}

char *get_cmd(char *msg)
{
	int i, len = strlen(msg), start = -1;
	char *cmd = (char *) calloc(50, sizeof(char));

	for(i = 0; i < len; i++) {
		if(msg[i] == '+') {
			start = i;
		} else if(start != -1 && msg[i] == ':') {
			strncpy(cmd, msg + start, i - start);
			cmd[i - start] = '\0';
			break;
		}
	}

	return cmd;
}

void emergency_alarm() 
{
	if(alarm_on) {
		emergency = 1;
	}
}

ISR(PCINT1_vect)
{
    if ((PINB & (1 << PB2)) == 0) {
		// Nothing
	} else {
		emergency_alarm();
	}
}

ISR(PCINT3_vect)
{
    if ((PIND & (1 << PD6)) == 0) {
		// Nothing
	} else {
		emergency = 0;
		alarm_on = 0;
		PORTD &= ~(1 << PD7);
	}
}

ISR(USART0_RX_vect)
{
	recv = 1;
	buf[pos] = USART0_receive();
	pos++;
}

int main()
{
	// Led
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);

	// Senzor
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);

	// Buton
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6);

	// Activez PIN CHANGE INTERRUPT ENABLE (portul B)
	PCICR |= (1 << PCIE1);
	// Activez PIN CHANGE INTERRUPT ENABLE (portul D)
	PCICR |= (1 << PCIE3);

    // Activez intreruperea pe butoanele PB2
    PCMSK1 |= (1 << PCINT10);
    // Activez intreruperea pe butoanele PD6
    PCMSK3 |= (1 << PCINT30);

    USART0_init();
    sei();
    _delay_ms(2000);
    // Init
 	USART0_print("AT\n\r");
 	_delay_ms(200);
 	USART0_print("AT\n\r");
 	_delay_ms(200);
 	USART0_print("AT\n\r");
 	_delay_ms(2000);
 	// Text mode
 	USART0_print("AT+CMGF=1\n\r");
 	// Delete all sms
 	USART0_print("AT+CMGD=1,4\n\r");

    char *msg, *cmd, *no;

	while(1) {
		_delay_ms(1000);

		if(emergency && alarm_on) {
			USART0_print("ATD");
			USART0_print(phoneNo);
			USART0_print("\n\r");
			_delay_ms(15000);
			USART0_print("ATH\n\r");
			_delay_ms(5000);
		}

		if(recv) {
			_delay_ms(1000);
			buf[pos] = '\0';
			cmd = get_cmd(buf);

			// If sms received
			if(strcmp(cmd, "+CIEV") == 0) {
				no = get_no(buf);
				msg = get_msg(buf);

				if(strcmp(no, phoneNo) == 0) {
					if(strcmp(msg, "alarm_on") == 0) {
						alarm_on = 1;
						PORTD |= (1 << PD7);
						emergency = 0;
					} else if(strcmp(msg, "alarm_off") == 0) {
						alarm_on = 0;
						emergency = 0;
						PORTD &= ~(1 << PD7);
					}
				}

				free(no);
				free(msg);

				// Delete messages for free storage
				USART0_print("AT+CMGD=1,4\n\r");
			}

			free(cmd);

			recv = 0;
			pos = 0;
		}
	}

	return 0;
}	